@@ -203,7 +203,7 @@ u32 sqlite3PagerDataVersion(Pager*);
   int sqlite3PagerRefcount(Pager*);
 #endif
 int sqlite3PagerMemUsed(Pager*);
-const char *sqlite3PagerFilename(Pager*, int);
+const char *sqlite3PagerFilename(const Pager*, int);
 sqlite3_vfs *sqlite3PagerVfs(Pager*);
 sqlite3_file *sqlite3PagerFile(Pager*);
 sqlite3_file *sqlite3PagerJrnlFile(Pager*);
