@@ -12,7 +12,11 @@
 ** Code for testing the SQLite library in a multithreaded environment.
 */
 #include "sqliteInt.h"
-#include "tcl.h"
+#if defined(INCLUDE_SQLITE_TCL_H)
+#  include "sqlite_tcl.h"
+#else
+#  include "tcl.h"
+#endif
 #if SQLITE_OS_UNIX && SQLITE_THREADSAFE
 #include <stdlib.h>
 #include <string.h>
@@ -60,7 +64,7 @@ static Thread threadset[N_THREAD];
 /*
 ** The main loop for a thread.  Threads use busy waiting. 
 */
-static void *thread_main(void *pArg){
+static void *test_thread_main(void *pArg){
   Thread *p = (Thread*)pArg;
   if( p->db ){
     sqlite3_close(p->db);
@@ -121,7 +125,7 @@ static int parse_thread_id(Tcl_Interp *interp, const char *zArg){
 ** NAME should be an upper case letter.  Start the thread running with
 ** an open connection to the given database.
 */
-static int tcl_thread_create(
+static int SQLITE_TCLAPI tcl_thread_create(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -147,7 +151,7 @@ static int tcl_thread_create(
   threadset[i].zFilename = sqlite3_mprintf("%s", argv[2]);
   threadset[i].opnum = 1;
   threadset[i].completed = 0;
-  rc = pthread_create(&x, 0, thread_main, &threadset[i]);
+  rc = pthread_create(&x, 0, test_thread_main, &threadset[i]);
   if( rc ){
     Tcl_AppendResult(interp, "failed to create the thread", 0);
     sqlite3_free(threadset[i].zFilename);
@@ -161,7 +165,7 @@ static int tcl_thread_create(
 /*
 ** Wait for a thread to reach its idle state.
 */
-static void thread_wait(Thread *p){
+static void test_thread_wait(Thread *p){
   while( p->opnum>p->completed ) sched_yield();
 }
 
@@ -170,7 +174,7 @@ static void thread_wait(Thread *p){
 **
 ** Wait on thread ID to reach its idle state.
 */
-static int tcl_thread_wait(
+static int SQLITE_TCLAPI tcl_thread_wait(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -189,18 +193,18 @@ static int tcl_thread_wait(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   return TCL_OK;
 }
 
 /*
 ** Stop a thread.
 */
-static void stop_thread(Thread *p){
-  thread_wait(p);
+static void test_stop_thread(Thread *p){
+  test_thread_wait(p);
   p->xOp = 0;
   p->opnum++;
-  thread_wait(p);
+  test_thread_wait(p);
   sqlite3_free(p->zArg);
   p->zArg = 0;
   sqlite3_free(p->zFilename);
@@ -214,7 +218,7 @@ static void stop_thread(Thread *p){
 ** Cause a thread to shut itself down.  Wait for the shutdown to be
 ** completed.  If ID is "*" then stop all threads.
 */
-static int tcl_thread_halt(
+static int SQLITE_TCLAPI tcl_thread_halt(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -229,7 +233,7 @@ static int tcl_thread_halt(
   }
   if( argv[1][0]=='*' && argv[1][1]==0 ){
     for(i=0; i<N_THREAD; i++){
-      if( threadset[i].busy ) stop_thread(&threadset[i]);
+      if( threadset[i].busy ) test_stop_thread(&threadset[i]);
     }
   }else{
     i = parse_thread_id(interp, argv[1]);
@@ -238,7 +242,7 @@ static int tcl_thread_halt(
       Tcl_AppendResult(interp, "no such thread", 0);
       return TCL_ERROR;
     }
-    stop_thread(&threadset[i]);
+    test_stop_thread(&threadset[i]);
   }
   return TCL_OK;
 }
@@ -249,7 +253,7 @@ static int tcl_thread_halt(
 ** Wait on the most recent thread_step to complete, then return the
 ** number of columns in the result set.
 */
-static int tcl_thread_argc(
+static int SQLITE_TCLAPI tcl_thread_argc(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -269,7 +273,7 @@ static int tcl_thread_argc(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   sqlite3_snprintf(sizeof(zBuf), zBuf, "%d", threadset[i].argc);
   Tcl_AppendResult(interp, zBuf, 0);
   return TCL_OK;
@@ -281,7 +285,7 @@ static int tcl_thread_argc(
 ** Wait on the most recent thread_step to complete, then return the
 ** value of the N-th columns in the result set.
 */
-static int tcl_thread_argv(
+static int SQLITE_TCLAPI tcl_thread_argv(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -302,7 +306,7 @@ static int tcl_thread_argv(
     return TCL_ERROR;
   }
   if( Tcl_GetInt(interp, argv[2], &n) ) return TCL_ERROR;
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   if( n<0 || n>=threadset[i].argc ){
     Tcl_AppendResult(interp, "column number out of range", 0);
     return TCL_ERROR;
@@ -317,7 +321,7 @@ static int tcl_thread_argv(
 ** Wait on the most recent thread_step to complete, then return the
 ** name of the N-th columns in the result set.
 */
-static int tcl_thread_colname(
+static int SQLITE_TCLAPI tcl_thread_colname(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -338,7 +342,7 @@ static int tcl_thread_colname(
     return TCL_ERROR;
   }
   if( Tcl_GetInt(interp, argv[2], &n) ) return TCL_ERROR;
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   if( n<0 || n>=threadset[i].argc ){
     Tcl_AppendResult(interp, "column number out of range", 0);
     return TCL_ERROR;
@@ -353,7 +357,7 @@ static int tcl_thread_colname(
 ** Wait on the most recent operation to complete, then return the
 ** result code from that operation.
 */
-static int tcl_thread_result(
+static int SQLITE_TCLAPI tcl_thread_result(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -373,7 +377,7 @@ static int tcl_thread_result(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   zName = sqlite3ErrName(threadset[i].rc);
   Tcl_AppendResult(interp, zName, 0);
   return TCL_OK;
@@ -385,7 +389,7 @@ static int tcl_thread_result(
 ** Wait on the most recent operation to complete, then return the
 ** error string.
 */
-static int tcl_thread_error(
+static int SQLITE_TCLAPI tcl_thread_error(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -404,7 +408,7 @@ static int tcl_thread_error(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   Tcl_AppendResult(interp, threadset[i].zErr, 0);
   return TCL_OK;
 }
@@ -430,7 +434,7 @@ static void do_compile(Thread *p){
 **
 ** Compile a new virtual machine.
 */
-static int tcl_thread_compile(
+static int SQLITE_TCLAPI tcl_thread_compile(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -448,7 +452,7 @@ static int tcl_thread_compile(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   threadset[i].xOp = do_compile;
   sqlite3_free(threadset[i].zArg);
   threadset[i].zArg = sqlite3_mprintf("%s", argv[2]);
@@ -483,7 +487,7 @@ static void do_step(Thread *p){
 **
 ** Advance the virtual machine by one step
 */
-static int tcl_thread_step(
+static int SQLITE_TCLAPI tcl_thread_step(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -501,7 +505,7 @@ static int tcl_thread_step(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   threadset[i].xOp = do_step;
   threadset[i].opnum++;
   return TCL_OK;
@@ -525,7 +529,7 @@ static void do_finalize(Thread *p){
 **
 ** Finalize the virtual machine.
 */
-static int tcl_thread_finalize(
+static int SQLITE_TCLAPI tcl_thread_finalize(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -543,7 +547,7 @@ static int tcl_thread_finalize(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   threadset[i].xOp = do_finalize;
   sqlite3_free(threadset[i].zArg);
   threadset[i].zArg = 0;
@@ -556,7 +560,7 @@ static int tcl_thread_finalize(
 **
 ** Interchange the sqlite* pointer between two threads.
 */
-static int tcl_thread_swap(
+static int SQLITE_TCLAPI tcl_thread_swap(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -575,14 +579,14 @@ static int tcl_thread_swap(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   j = parse_thread_id(interp, argv[2]);
   if( j<0 ) return TCL_ERROR;
   if( !threadset[j].busy ){
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[j]);
+  test_thread_wait(&threadset[j]);
   temp = threadset[i].db;
   threadset[i].db = threadset[j].db;
   threadset[j].db = temp;
@@ -596,7 +600,7 @@ static int tcl_thread_swap(
 ** remove the pointer from the thread itself.  Afterwards, the thread
 ** can be stopped and the connection can be used by the main thread.
 */
-static int tcl_thread_db_get(
+static int SQLITE_TCLAPI tcl_thread_db_get(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -616,7 +620,7 @@ static int tcl_thread_db_get(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   sqlite3TestMakePointerStr(interp, zBuf, threadset[i].db);
   threadset[i].db = 0;
   Tcl_AppendResult(interp, zBuf, (char*)0);
@@ -627,7 +631,7 @@ static int tcl_thread_db_get(
 ** Usage: thread_db_put ID DB
 **
 */
-static int tcl_thread_db_put(
+static int SQLITE_TCLAPI tcl_thread_db_put(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -647,7 +651,7 @@ static int tcl_thread_db_put(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   assert( !threadset[i].db );
   threadset[i].db = (sqlite3*)sqlite3TestTextToPtr(argv[2]);
   return TCL_OK;
@@ -659,7 +663,7 @@ static int tcl_thread_db_put(
 ** Return the database stmt pointer for the given thread.  Then
 ** remove the pointer from the thread itself. 
 */
-static int tcl_thread_stmt_get(
+static int SQLITE_TCLAPI tcl_thread_stmt_get(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -679,7 +683,7 @@ static int tcl_thread_stmt_get(
     Tcl_AppendResult(interp, "no such thread", 0);
     return TCL_ERROR;
   }
-  thread_wait(&threadset[i]);
+  test_thread_wait(&threadset[i]);
   sqlite3TestMakePointerStr(interp, zBuf, threadset[i].pStmt);
   threadset[i].pStmt = 0;
   Tcl_AppendResult(interp, zBuf, (char*)0);
