@@ -39,8 +39,14 @@ LEAN_OPTS="$LEAN_OPTS -DSQLITE_USE_ALLOCA"
 BASELINE="trunk"
 doExplain=0
 doCachegrind=1
+doVdbeProfile=0
+doWal=1
+doDiff=1
 while test "$1" != ""; do
   case $1 in
+    --nodiff)
+	doDiff=0
+        ;;
     --reprepare)
         SPEEDTEST_OPTS="$SPEEDTEST_OPTS $1"
         ;;
@@ -59,11 +65,23 @@ while test "$1" != ""; do
     --nomemstat)
         SPEEDTEST_OPTS="$SPEEDTEST_OPTS $1"
         ;;
+    --multithread)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS $1"
+        ;;
+    --singlethread)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS $1"
+        ;;
+    --serialized)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS $1"
+        ;;
     --temp)
         SPEEDTEST_OPTS="$SPEEDTEST_OPTS --temp 6"
         ;;
+    --legacy)
+	doWal=0
+        ;;
     --wal)
-        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --journal wal"
+        doWal=1
         ;;
     --size)
         shift; SIZE=$1
@@ -78,6 +96,7 @@ while test "$1" != ""; do
         rm -f vdbe_profile.out
         CC_OPTS="$CC_OPTS -DVDBE_PROFILE"
         doCachegrind=0
+        doVdbeProfile=1
         ;;
     --lean)
         CC_OPTS="$CC_OPTS $LEAN_OPTS"
@@ -132,11 +151,17 @@ while test "$1" != ""; do
   esac
   shift
 done
+if test $doWal -eq 1; then
+  SPEEDTEST_OPTS="$SPEEDTEST_OPTS --journal wal"
+fi
 SPEEDTEST_OPTS="$SPEEDTEST_OPTS --size $SIZE"
 echo "NAME           = $NAME" | tee summary-$NAME.txt
 echo "SPEEDTEST_OPTS = $SPEEDTEST_OPTS" | tee -a summary-$NAME.txt
 echo "CC_OPTS        = $CC_OPTS" | tee -a summary-$NAME.txt
 rm -f cachegrind.out.* speedtest1 speedtest1.db sqlite3.o
+if test $doVdbeProfile -eq 1; then
+  rm -f vdbe_profile.out
+fi
 $CC -g -Os -Wall -I. $CC_OPTS -c sqlite3.c
 size sqlite3.o | tee -a summary-$NAME.txt
 if test $doExplain -eq 1; then
@@ -163,6 +188,10 @@ fi
 if test $doExplain -eq 1; then
   ./speedtest1 --explain $SPEEDTEST_OPTS | ./sqlite3 >explain-$NAME.txt
 fi
-if test "$NAME" != "$BASELINE"; then
+if test $doVdbeProfile -eq 1; then
+  tclsh ../sqlite/tool/vdbe_profile.tcl >vdbeprofile-$NAME.txt
+  open vdbeprofile-$NAME.txt
+fi
+if test "$NAME" != "$BASELINE" -a $doVdbeProfile -ne 1 -a $doDiff -ne 0; then
   fossil test-diff --tk -c 20 cout-$BASELINE.txt cout-$NAME.txt
 fi
